@@ -1,6 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from __future__ import division
+
 
 from itertools import groupby
 from collections import defaultdict
@@ -30,8 +30,7 @@ INVOICE_METHODS = [
     ]
 
 
-class Work:
-    __metaclass__ = PoolMeta
+class Work(metaclass=PoolMeta):
     __name__ = 'project.work'
     project_invoice_method = fields.Selection(INVOICE_METHODS,
         'Invoice Method',
@@ -76,6 +75,8 @@ class Work:
                         | (Eval('project_invoice_method', 'manual')
                             == 'manual')),
                     'readonly': ~Eval('duration_to_invoice'),
+                    'depends': ['type', 'project_invoice_method',
+                        'duration_to_invoice']
                     },
                 })
         cls._error_messages.update({
@@ -92,7 +93,8 @@ class Work:
     def copy(cls, records, default=None):
         if default is None:
             default = {}
-        default = default.copy()
+        else:
+            default = default.copy()
         default.setdefault('invoice_line', None)
         return super(Work, cls).copy(records, default=default)
 
@@ -243,7 +245,7 @@ class Work:
                     where=where))
             work2currency.update(cursor.fetchall())
 
-        currencies = Currency.browse(set(work2currency.itervalues()))
+        currencies = Currency.browse(set(work2currency.values()))
         id2currency = {c.id: c for c in currencies}
 
         for work in works:
@@ -292,7 +294,7 @@ class Work:
                     where=where))
             work2currency.update(cursor.fetchall())
 
-        currencies = Currency.browse(set(work2currency.itervalues()))
+        currencies = Currency.browse(set(work2currency.values()))
         id2currency = {c.id: c for c in currencies}
 
         for work in works:
@@ -313,8 +315,7 @@ class Work:
 
         durations = defaultdict(datetime.timedelta)
         twork2work = {tw.id: w.id for w in works for tw in w.timesheet_works}
-        ids = twork2work.keys()
-        for sub_ids in grouped_slice(ids):
+        for sub_ids in grouped_slice(twork2work.keys()):
             red_sql = reduce_ids(line.work, sub_ids)
             if invoiced:
                 where = line.invoice_line != Null
@@ -338,7 +339,7 @@ class Work:
         method2works = defaultdict(list)
         for work in works:
             method2works[work.invoice_method].append(work)
-        for method, m_works in method2works.iteritems():
+        for method, m_works in method2works.items():
             method = getattr(cls, '_get_%s_%s' % (name, method))
             # Re-browse for cache alignment
             durations.update(method(cls.browse(m_works)))
@@ -363,8 +364,11 @@ class Work:
         Invoice = pool.get('account.invoice')
 
         invoices = []
-        for work in works:
-            invoice_lines = work._get_lines_to_invoice()
+        uninvoiced = works[:]
+        while uninvoiced:
+            work = uninvoiced.pop(0)
+            invoice_lines, uninvoiced_children = work._get_lines_to_invoice()
+            uninvoiced.extend(uninvoiced_children)
             if not invoice_lines:
                 continue
             invoice = work._get_invoice()
@@ -381,7 +385,7 @@ class Work:
                 for line in lines:
                     origin = line['origin']
                     origins.setdefault(origin.__class__, []).append(origin)
-                for klass, records in origins.iteritems():
+                for klass, records in origins.items():
                     klass.save(records)  # Store first new origins
                     klass.write(records, {
                             'invoice_line': invoice_line.id,
@@ -524,7 +528,6 @@ class Work:
                     'unit_price': self.list_price,
                     'origin': invoiced_progress,
                     'description': self.name,
-                    'description': self.name,
                     }]
         return []
 
@@ -561,14 +564,18 @@ class Work:
         lines = []
         if test is None:
             test = self._test_group_invoice()
+        uninvoiced_children = []
         lines += getattr(self, '_get_lines_to_invoice_%s' %
             self.invoice_method)()
         for children in self.children:
             if children.type == 'project':
                 if test != children._test_group_invoice():
+                    uninvoiced_children.append(children)
                     continue
-            lines += children._get_lines_to_invoice(test=test)
-        return lines
+            child_lines, uninvoiced = children._get_lines_to_invoice(test=test)
+            lines.extend(child_lines)
+            uninvoiced_children.extend(uninvoiced)
+        return lines, uninvoiced_children
 
 
 class WorkInvoicedProgress(ModelView, ModelSQL):
